@@ -19,7 +19,7 @@
 ## You should have received a copy of the GNU General Public License
 ## along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import pprint, sys, csv, math, inspect
+import pprint, sys, csv, math, inspect, os
 
 from .settings import settings
 from .util import cum_prob, frange
@@ -58,12 +58,16 @@ class Formatter(object):
             if output == "-":
                 self.output = sys.stdout
             else:
-                self.output = open(output, "w")
+                try:
+                    self.output = open(output, "w")
+                except IOError as e:
+                    raise RuntimeError("Unable to output data: %s" % e)
         else:
             self.output = output
 
     def format(self, results):
-        sys.stderr.write("No output formatter selected.\nTest data is in %s (use with -i to format).\n" % results[0].dump_file)
+        if results[0].dump_file is not None:
+            sys.stderr.write("No output formatter selected.\nTest data is in %s (use with -i to format).\n" % results[0].dump_file)
 
 DefaultFormatter = Formatter
 
@@ -182,6 +186,8 @@ class StatsFormatter(Formatter):
 class PlotFormatter(Formatter):
 
     def __init__(self, output):
+        if output != "-" and not os.access(output, os.W_OK):
+            raise RuntimeError("No write permission for output file '%s'" % output)
         self.output = output
         try:
             import matplotlib, numpy
@@ -299,6 +305,8 @@ class PlotFormatter(Formatter):
             data.append([])
 
         for s in config['series']:
+            if not s['data'] in results.series_names:
+                continue
             if 'smoothing' in s:
                 smooth=s['smoothing']
             else:
@@ -329,7 +337,8 @@ class PlotFormatter(Formatter):
             btm,top = 0,100
 
         for a in range(len(config['axes'])):
-            self._do_scaling(config['axes'][a], data[a], btm, top)
+            if data[a]:
+                self._do_scaling(config['axes'][a], data[a], btm, top)
 
     def _do_cdf_plot(self, results, config=None, axis=None, postfix=""):
         if axis is None:
@@ -341,6 +350,9 @@ class PlotFormatter(Formatter):
         sizes = []
         max_value = 0.0
         for s in config['series']:
+            if not s['data'] in results.series_names:
+                data.append([])
+                continue
             s_data = results.series(s['data'])
             if 'cutoff' in config:
                 # cut off values from the beginning and end before doing the
@@ -380,7 +392,7 @@ class PlotFormatter(Formatter):
                       [cum_prob(data[i], point, sizes[i]) for point in x_values],
                       **kwargs)
 
-        if max(self.medians)/min(self.medians) > 10.0:
+        if self.medians and max(self.medians)/min(self.medians) > 10.0:
             # More than an order of magnitude difference; switch to log scale
             axis.set_xscale('log')
             min_val = min(self.min_vals)
@@ -423,7 +435,10 @@ class PlotFormatter(Formatter):
                 a.set_position([box.x0, box.y0, box.width * 0.8, box.height])
             self.plt.show()
         else:
-            self.plt.savefig(self.output, bbox_extra_artists=artists, bbox_inches='tight')
+            try:
+                self.plt.savefig(self.output, bbox_extra_artists=artists, bbox_inches='tight')
+            except IOError as e:
+                raise RuntimeError("Unable to save output plot: %s" % e)
 
 
     def _annotate_plot(self, skip_title=False):
